from .dcodefuncs import *
from .encodefuncs import *

__title__ = "dcoder"
__summary__ = (
    "dcoder is a module that can decode/encode text in various ciphers."
)
__uri__ = "https://github.com/CodeWithSwastik/dcoder"

__version__ = "0.2.0"

__author__ = "Swas.py"
__email__ = "cwswas.py@gmail.com"

__license__ = "MIT License"
__copyright__ = "Copyright 2021 {}".format(__author__)


def get_version():
    return __version__
